import pytest
from unittest.mock import Mock
from rest_framework.response import Response
from sparky_utils.exceptions import (
    handle_internal_server_exception,
)
import logging


def test_handle_internal_server_exception():

    # Act
    response = handle_internal_server_exception()

    # Assert
    assert isinstance(response, Response)
    assert response.status_code == 500
    assert response.data == {"status": "error", "message": "Something went wrong"}



import logging
import logging.config
from pathlib import Path
from dataclasses import dataclass
from django.utils.log import DEFAULT_LOGGING
import traceback


@dataclass
class LoggerConfig:
    log_level: str = logging.INFO
    log_format: str = "%(asctime)s [%(levelname)s] %(name)s - %(message)s"
    log_date_format: str = "%Y-%m-%d %H:%M:%S"
    base_dir: str = Path.cwd()

    def __post_init__(self):
        self.__configure_logger()

    def __configure_logger(self):
        try:
            logging.config.dictConfig(
                {
                    "version": 1,
                    "disable_existing_loggers": False,
                    "formatters": {
                        "console": {
                            "format": self.log_format,
                        },
                        "file": {
                            "format": self.log_format,
                        },
                        "django.server": DEFAULT_LOGGING["formatters"]["django.server"],
                    },
                    "handlers": {
                        "console": {
                            "level": self.log_level,
                            "class": "logging.StreamHandler",
                            "formatter": "console",
                        },
                        "file": {
                            "level": self.log_level,
                            "class": "logging.handlers.RotatingFileHandler",
                            "formatter": "file",
                            "filename": self.base_dir / "logs" / "app.log",
                            "maxBytes": 1024 * 1024 * 10,  # 5 MB
                            "backupCount": 5,
                        },
                        "django.server": DEFAULT_LOGGING["handlers"]["django.server"],
                    },
                    "loggers": {
                        "": {
                            "level": self.log_level,
                            "handlers": ["console", "file"],
                            "propagate": False,
                        },
                        "apps": {
                            "level": self.log_level,
                            "handlers": ["console", "file"],
                            "propagate": False,
                        },
                    },
                    "django.server": DEFAULT_LOGGING["loggers"]["django.server"],
                }
            )

        except ValueError:
            print(f"Log directory '{self.base_dir / 'logs'}' not found.")
            print("Creating directory...")
            (self.base_dir / "logs").mkdir(parents=True, exist_ok=True)
            self.__configure_logger()

        except Exception:
            print(f"Failed to load logging configuration: {traceback.format_exc()}")



import pytest
import sys
import os
from rest_framework.response import Response
from sparky_utils.response import service_response
from django.conf import settings

settings.configure()


sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))


def test_service_response_with_empty_data():
    expected_status = "success"
    expected_message = ""
    expected_data = {}
    expected_status_code = 200

    response = service_response(status=expected_status, message=expected_message)

    assert isinstance(response, Response)
    assert response.status_code == expected_status_code
    assert response.data == {
        "status": expected_status,
        "message": expected_message,
        "data": expected_data,
    }


def test_service_response_with_empty_data_and_custom_status():
    expected_status = "custom_status"
    expected_message = ""
    expected_data = {}
    expected_status_code = 200

    response = service_response(status=expected_status, message=expected_message)

    assert isinstance(response, Response)
    assert response.status_code == expected_status_code
    assert response.data == {
        "status": expected_status,
        "message": expected_message,
        "data": expected_data,
    }


def test_service_response_with_empty_data_and_custom_status_code():
    expected_status = "success"
    expected_message = ""
    expected_data = {}
    expected_status_code = 404

    response = service_response(
        status=expected_status,
        message=expected_message,
        status_code=expected_status_code,
    )

    assert isinstance(response, Response)
    assert response.status_code == expected_status_code
    assert response.data == {
        "status": expected_status,
        "message": expected_message,
        "data": expected_data,
    }


def test_service_response_with_empty_data_and_custom_status_and_status_code():
    expected_status = "error"
    expected_message = ""
    expected_data = {}
    expected_status_code = 404

    response = service_response(
        status=expected_status,
        message=expected_message,
        status_code=expected_status_code,
    )

    assert isinstance(response, Response)
    assert response.status_code == expected_status_code
    assert response.data == {
        "status": expected_status,
        "message": expected_message,
        "data": expected_data,
    }


def test_service_response_with_empty_data_and_message():
    expected_status = "success"
    expected_message = "Custom message"
    expected_data = {}
    expected_status_code = 200

    response = service_response(status=expected_status, message=expected_message)

    assert isinstance(response, Response)
    assert response.status_code == expected_status_code
    assert response.data == {
        "status": expected_status,
        "message": expected_message,
        "data": expected_data,
    }



import logging.config
from sparky_utils.logger import LoggerConfig
import logging
from pathlib import Path
from logging.config import dictConfig

base_dir = Path(__file__).resolve().parent.parent

logger_config = LoggerConfig(log_level=logging.INFO)


def test_logger_config_log_level():
    assert logger_config.log_level == logging.INFO


def test_logger_config_log_file_path():
    assert str(logger_config.base_dir) == str(base_dir) + "\sparky_utils"


def test_logger_config_an_instance_of_logging_config():
    assert isinstance(logger_config, LoggerConfig)


def test_logger_info():
    logger = logging.getLogger(__name__)
    logger.info("This is a test info message")
    assert hasattr(logger, "handlers")


def test_logger_correctly_log(caplog):
    logger = logging.getLogger(__name__)
    with caplog.at_level(logging.INFO):
        logger.info("This is a test info message")

    # Check if the message was logged
    assert (
        "This is a test info message" in caplog.text
    ), "The log message was not captured"

    assert len(caplog.records) == 1, "Unexpected number of log records"
    assert caplog.records[0].levelname == "INFO", "The log level is incorrect"
    assert (
        caplog.records[0].message == "This is a test info message"
    ), "The log message content is incorrect"



import logging
import traceback
from rest_framework.response import Response

logger = logging.getLogger(__name__)


def handle_internal_server_exception() -> Response:
    """Handles internal Server error exception

    Returns:
        Response: Http response
    """
    traceback.print_exc()
    logger.error(traceback.format_exc())
    return Response({"status": "error", "message": "Something went wrong"}, status=500)


class ServiceException(Exception):
    def __init__(self, status_code, message):
        super().__init__(message)
        self.status_code = status_code
        self.message = message


import pkgutil
import importlib


def list_modules():
    """List all modules in the current package."""
    package_name = __name__.rsplit(".", 1)[0]
    package = importlib.import_module(package_name)

    module_list = []
    if hasattr(package, "__path__"):
        for module_info in pkgutil.iter_modules(package.__path__):
            module_list.append(module_info.name)

    return module_list



from rest_framework.response import Response


def service_response(
    status: str = "success", data: dict = {}, message: str = "", status_code=200
) -> Response:
    """Api response utils functions

    Args:
        status (str, optional): request status Defaults to "success".
        data (dict, optional): response data. Defaults to None.
        message (str, optional): response message. Defaults to None.
        status (int, optional): response status code. Defaults to 200.

    Returns:
        Response: Http response
    """
    return Response(
        {"status": status, "message": message, "data": data, "status_code": status_code}, status=status_code
    )



def transform_name(name):
    result = ""
    for char in name:
        if char.isupper() and result:
            result += " "
        result += char
    return result


def str_meta(cls):
    """
    Custom decorator to automatically add __str__ method and Meta class to Django models.
    """

    def __str__(self):
        return getattr(self, "name", str(self.pk))

    cls.__str__ = __str__

    class Meta:
        verbose_name_plural = transform_name(cls.__name__) + "s"
        ordering = ["name"]
        verbose_name = transform_name(cls.__name__)
        db_table = cls.__name__.lower()

    cls.Meta = Meta

    return cls



from setuptools import setup, find_packages

setup(
    name="sparky_utils",
    version="1.1.0",
    author="Ayobami Alaran",
    author_email="ayobamidele006@gmail.com",
    description="Python and Django Project Utils Package",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/Ayobami6/sparky_utils",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
    install_requires=[
        "asgiref==3.8.1",
        "Django>=4.2.13",
        "djangorestframework>=3.15.1",
    ],
)



from .response import service_response
from .exceptions import handle_internal_server_exception, ServiceException
from functools import wraps
from django.core.exceptions import ValidationError, ObjectDoesNotExist


def exception_advice(func) -> any: 
    @wraps(func)
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except ServiceException as e:
            return service_response(status="error", message=str(e), status_code=e.status_code)
        except ValidationError as e:
            return service_response(status="error", message=str(e), status_code=400)
        except ObjectDoesNotExist as e:
            return service_response(status="error", message=str(e), status_code=404)
        except Exception:
            return handle_internal_server_exception()
    
    return wrapper